/*global define*/
define([
  'jquery',
  'underscore',
  'backbone',
  'views/content',
  'context',
  'baseview',
  'events',
  'views/user/account',
  'views/user/license',
  'views/user/team',
  'models/license'
], function ($, _, Backbone, ContentView, Context, BaseView, Events,
             AccountView, LicenseView, TeamView, License) {
  'use strict';

  var AppView = BaseView.extend({
    el: '#content-app',
    initialize: function () {
      console.log("appview init", Context);
      this.listenTo(Events, "overlay:close", function () {
        //this.$(".overlay-wrapper").html("");
        //this.$(".main-wrapper").show();
      });

      this.listenTo(Events, "route:user:team", function () {
        // initialize team view
        this.$(".main-wrapper").hide();
        this.subviews["teamView"] = new TeamView();
        this.$(".overlay-wrapper").html(this.subviews["teamView"].render().el);
      });

      this.listenTo(Events, "route:user:account", function () {
        // initialize account view
        this.$(".main-wrapper").hide();
        this.subviews["accountView"] = new AccountView();
        this.$(".overlay-wrapper").html(this.subviews["accountView"].render().el);
      });

      this.listenTo(Events, "route:user:license", function () {
        // initialize account view
        this.$(".main-wrapper").hide();
        this.subviews["licenseView"] = new LicenseView();
        this.$(".overlay-wrapper").html(this.subviews["licenseView"].render().el);
      });

      // license event listener
      this.listenTo(Events, "license:check", function () {

        // fetch and check license
        var license = new License();
        license.fetch({
          success: function (model) {
            License.setLicense(model);
            console.info("[license] fetch->success", model.toJSON());
            if (!model.isValid() && Backbone.history.fragment !== "license"
                && Backbone.history.fragment !== "team") {
              alert(model.getErrorMessage());
              window.location = "#/license";
            }
          },
          error: function () {
            // failed to fetch license
            console.warn("[license] fetch->error", arguments);
          }
        });

      });

    },
    render: function () {
      console.log("appview render", Context);

      //if (this.subviews["app"]) {
      //    // remove app
      //    this.subviews["app"].remove();
      //
      //    // unbind global events
      //    Events.unbind();
      //}

      // initialize main content view
      this.subviews["app"] = new ContentView();
      this.$el.html(this.subviews["app"].render().el);

    }
  });

  return AppView;
});