var context = context || {};
var contextStorage = {}

String.prototype.format = function (a) {
    return sprintf(this, a);
};
String.prototype.toTitle = function () {
    var str = this.replace(/_/g, ' ');
    if (str.length > 0) {
        return str.charAt(0).toUpperCase() + str.slice(1);
    }
    return "";
};
Array.prototype.first = function () {
    if (this.length > 0) {
        return this[0];
    }
    return null;
};

$.fn.redraw = function () {
    return this.hide(0, function () {
        $(this).show()
    });
};

$.fn.showErrors = function (errors) {
    var $form = $(this);
    $form.find('.has-error').removeClass('has-error');
    $.each(errors, function (i, error) {
        $form.find("input[name=" + i + "]").parents('.form-group').addClass('has-error');
    });
};

$.fn.addAlert = function (msg) {
    $(this).html($("<div>").addClass("alert alert-danger").text(msg));
};

function getParameterByName(name) {
    name = name.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
    var regex = new RegExp("[\\?&]" + name + "=([^&#]*)"),
        results = regex.exec(location.search);
    return results == null ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
}

function isNumber(n) {
    return !isNaN(parseFloat(n)) && isFinite(n);
}

function getCookie(name) {
    var cookieValue = null;
    if (document.cookie && document.cookie != '') {
        var cookies = document.cookie.split(';');
        for (var i = 0; i < cookies.length; i++) {
            var cookie = jQuery.trim(cookies[i]);
            // Does this cookie string begin with the name we want?
            if (cookie.substring(0, name.length + 1) == (name + '=')) {
                cookieValue = decodeURIComponent(cookie.substring(name.length + 1));
                break;
            }
        }
    }
    return cookieValue;
}

function csrfSafeMethod(method) {
    // these HTTP methods do not require CSRF protection
    return (/^(GET|HEAD|OPTIONS|TRACE)$/.test(method));
}

function render(tmpl_name, templ_id, tmpl_data) {
    tmpl_data = tmpl_data || {};

    if (!render.tmpl_cache) {
        render.tmpl_cache = {};
    }

    if (!render.tmpl_cache[tmpl_name]) {
        var timestamp = new Date().getTime();
        var tmpl_dir = '/static/templates';
        var tmpl_url = tmpl_dir + '/' + tmpl_name + '.hbs?' + timestamp;
        var tmpl_string;

        $.ajax({
            url: tmpl_url,
            method: 'GET',
            async: false,
            success: function (data) {
                tmpl_string = data;
            }
        });
    }

    tmpl_data['context'] = context;

    var template = Handlebars.compile($(tmpl_string).filter(templ_id).get(0).innerHTML);
    return template(tmpl_data);
}

var openView = function (where, view_id, tmpl_name, tmpl_id, tmpl_data, cbk) {
    var $where = $(where);
    var data_attr = "[data-view=" + view_id + "]";
    var $el = $where.find(data_attr);

    // check if template already is loaded
    if ($el.length > 0 && $el.is(':hidden')) {
        $where.find("div[data-view]").hide().removeClass('active-view');
        $el.show().addClass('active-view');
    }

    // create view
    if ($el.length == 0) {
        $where.find("div[data-view]").hide().removeClass('active-view');
        var $newCont = $("<div data-view='" + view_id + "'>").addClass('active-view');
        var $renderEl = $newCont.appendTo($where);
        $renderEl.html(render(tmpl_name, tmpl_id, tmpl_data));
        if (cbk) cbk();

        // we need to store context
        console.info("create storage for", view_id, context);
        contextStorage[view_id] = $.extend({}, context); // clone instead of passing a reference
    }
};

var heightUpdateFunction = function () {
    var bottomPadding = editor.renderer.lineHeight;
    var newHeight =
        editor.getSession().getScreenLength()
            * editor.renderer.lineHeight
            + editor.renderer.scrollBar.getWidth()
            + bottomPadding;

    $('#editor').height(newHeight.toString() + "px");
    //$('#editor-section').height(newHeight.toString() + "px");
    editor.resize();
    $(".main-content").redraw();
};

var updateChart = function (e) {
    datazen.drawChart($(".main-content .chart"), datazen.data);
    e.preventDefault();
}

function showTooltip(x, y, contents) {
    $("<div class='tooltip'><div class='tooltip-inner'>" + contents + "</div></div>").css({
        position: "absolute",
        display: "none",
        top: y + 7,
        left: x + 7,
        opacity: 0.90
    }).appendTo("body").fadeIn(200);
}

function isEmptyRow(instance, row) {
    var rowData = instance.getData()[row];
    for (var i = 0, ilen = rowData.length; i < ilen; i++) {
        if (rowData[i] !== null) {
            return false;
        }
    }
    return true;
}


var editor;
var datazen = {data: null, column_count: 0};

datazen.getChartSettings = function getChartSettings() {
    var settings = {};
    var keys = ["stack", "line", "curvedLine", "fill", "bar", "barWidth", "steps", "points", "legend", "lineWidth",
        "pointsSize", "innerRadius", "legendCols", "pie", "fillOpacity", "labelField", "lineSteps"]
    for (var i in keys) {
        var key = keys[i].charAt(0).toUpperCase() + keys[i].slice(1)
        var $el = $(".active-view").find("[data-setting=" + key + "]");
        if ($el.is(":checkbox")) {
            settings[keys[i]] = $el.is(":checked");
        } else {
            settings[keys[i]] = $el.val();
        }
    }
    return settings;
}

datazen.getQueryData = function (cbk, formatForChart) {
    formatForChart = formatForChart || false;
    var query = datazen.getQuery();
    console.info("query", query);
    if (query.length > 0) {
        $.post("/servers/" + context.server_id + "/query/" + context.database, {q: query, table: context.table, ffc: formatForChart}, function (res) {
            datazen.data = res;
            if (res.hasOwnProperty('error') && res.error) {
                $(".query-error").show().find(".error-text").text(res.error);
            } else {
                $(".query-error").hide();
                cbk(res);
            }
        });
    }
};

datazen.pageLimit = 50;
datazen.page = 1;
datazen.getData = function () {
    return datazen.data["rows"] || [];
}
datazen.getPrimaryKeys = function () {
    if(datazen.data.meta.primary_keys.length > 0) {
        return datazen.data.meta.primary_keys;
    } else {
        return this.getColumns();
    }
};
datazen.getModConditions = function (data) {
    var keys = datazen.getPrimaryKeys();
    var conditions = {};
    keys.forEach(function (key) {
        conditions[key] = data[key];
    });
    return conditions;
};

/** HS settings & functions */
datazen.getHS = function () {
    return $(".active-view .handsontable:first").handsontable('getInstance');
}
datazen.HSActiveNewRow = false;
datazen.setHSUnsavedRowStatus = function (index) {
    console.log("setHSUnsavedRowStatus", index);
    var $row = datazen.getHSRow(index);
    $row.data("unsaved", true);
};
datazen.getHSRow = function (index) {
    var hs = datazen.getHS();
    console.log("getCell", index, 0);
    var cell = hs.getCell(index, 0);
    return $(cell).parent();
}


datazen.getUpdateURL = function () {
    return "/servers/%(server_id)s/update/%(database)s/%(table)s".format(context);
}
datazen.getInsertURL = function () {
    return "/servers/%(server_id)s/insert/%(database)s/%(table)s".format(context);
}

datazen.tableView = function ($content) {
    this.getQueryData(function (res) {
        var rendered_html = render('content', "#query-table", res);
        $content.html(rendered_html);

        var cols = [], colHeaders = []
//        datazen.data["rows"][0].forEach(function (item, k) {
//            cols.push({data: k});
//        });

        // @fixme implement proper logic to get real columns or aliases
        // even when there are 0 rows

        datazen.getColumns().forEach(function (col) {
            cols.push({data: col});
        });

        $content.find(".handsontable").handsontable({
            data: datazen.getData(),
            contextMenu: false,
            stretchH: 'all',
            autoWrapRow: true,
            autoWrapCol: true,
            undo: false,
            columnSorting: true,
            currentRowClassName: 'currentRow',
            columns: cols,
            nativeScrollbars: false,
            fillHandle: false,
            minSpareRows: 1,
            colHeaders: datazen.getColumns(),
            afterSelectionEnd: function (row, col, row2, col2) {
                console.log("active row: ", row);
                console.log("HSActiveNewRow: ", datazen.HSActiveNewRow);
                console.log("insert mode:", !!datazen.HSActiveNewRow, datazen.HSActiveNewRow != row)
                // if there is active new row check if we need to save it
                if(datazen.HSActiveNewRow && datazen.HSActiveNewRow != row) {
                    console.log("save new row");
                    var data = this.getDataAtRow(datazen.HSActiveNewRow);
                    console.log("insert data:", data);
                    $.post(datazen.getInsertURL(), {data: JSON.stringify(data)}, function (res) {
                        console.log("insert request:", res);
                        if(res.status) {
                            // success
                        } else {
                            datazen.showError("Unable to insert row. " + res.error);
                        }
                    }, "json");
                    // unset active new row
                    datazen.HSActiveNewRow = false;
                }
            },
            afterInit: function () {
                // workaround for afterCreateRow bug
                // first afterCreateRow is called when table is not fully loaded, so we can't
                // access the newly created cell
                console.log("hs afterInit lastNewRow", this.lastNewRow);
//                datazen.setHSUnsavedRowStatus(this.lastNewRow);
                this.isFullyLoaded = true;
            },
            afterCreateRow: function (index, amount) {
                console.log("create new row:", index, amount);
                this.lastNewRow = index;
                // only set unsaved row here, when table is loaded
                if(this.isFullyLoaded) {
                    datazen.setHSUnsavedRowStatus(index);
                }
            },
            afterChange: function (changes, source) {
                console.log(changes, source);
                if (source === "edit") {
                    changes.forEach(function (item) {
                        var data = {row: item[0], prop: item[1], oldValue: item[2], newValue: item[3]};
                        if (data.oldValue !== data.newValue) {
                            // if this row in "unsaved" one, it can't be saved yet
                            // this row can be saved only when all values are filled
                            // and when this row loses focus - afterSelectionEnd
                            if(!datazen.getHSRow(data.row).data("unsaved")) {
//                                data.prikey = datazen.getPrimaryKey();
//                                data.prikeyValue = datazen.getHS().getDataAtRowProp(data.row, data.prikey);
                                var rowData = datazen.getHS().getDataAtRow(data.row);
                                console.log("rowData:", rowData);

                                data.conditions = datazen.getModConditions(rowData);
                                console.log("save data:", data);
                                $.post(datazen.getUpdateURL(), data, function (res) {
                                    console.log("update request:", res);
                                    if (res.status) {
                                        // success
                                    } else {
                                        datazen.showError("Unable to update row. " + res.error);
                                    }
                                }, "json");
                            } else {
                                datazen.HSActiveNewRow = data.row;
                            }
                        }
                    });
                }
            }
        });
//        $(".active-view .main-content").redraw();
    });
};
datazen.vizView = function ($content) {
    this.getQueryData(function (res) {

//        res['chartColumns'] = datazen.getChartColumns();

        // fetch chart template for the first time
        if ($(".chart:visible").length === 0) {
            res.cols = datazen.getColumns();
            var rendered_html = render('content', "#query-viz", res);
            $content.html(rendered_html);
        }

        // chart wrapper
        var $el = $content.find(".chart");

        // update colors
        datazen.updateColors();

        // update plottable columns
        var series_html = render('content', "#chart-series-column", {series: datazen.getChartColumns()});
        $(".chart-series-content").html(series_html);

        // set default chart title
        $("#chartTitle").val(context.table.toTitle());

        // draw chart
        datazen.drawChart($el, res);

        $(".main-content").redraw();
    });
};
datazen.getChartColumns = function () {
    if (datazen.data && datazen.data['rows'].length > 0) {
        return $.map(datazen.data['rows'][0], function (item, k) {
            if (isNumber(item)) return k;
        });
    }
    return [];
};
datazen.getColumnCount = function () {
    return datazen.getChartColumns().length
};
datazen.getColumns = function () {
    if (this.data && this.data.meta) {
        return this.data.meta.columns;
    }
    return [];
};
datazen.updateColors = function () {

    if($(".chart-colors .input-group").length > 0) {
        // only update colors for the first time
        return;
    }

    // default color scheme
    var base_colors = ["#3E2C52", "#8EB746", "#D3D3D3",
        "#E84B0C", "#FFFD00"];

    // get chart column count
    var needed_count = base_colors.length;

    var bc_length = base_colors.length;
    context.colors = [];

	$(".chart-colors").html("");
	for (var i = 0; i < needed_count; ++i) {
		context.colors.push(base_colors[i % bc_length]);
		var tpl = render("content", "#custom-color", {color: base_colors[i % bc_length]});
		$(".chart-colors").append(tpl);
	}

    context.colors = datazen.getColors();

    $(".colorpicker-component").colorpicker();
};
datazen.getColors = function () {
    return $(".colorpicker-component input").map(function () {
        return $(this).val();
    });
};
datazen.drawChart = function ($el, res) {

    var settings = datazen.getChartSettings();
//    settings.barWidth = 0.8;
    settings.fillColor = { colors: [
        { opacity: parseFloat(settings.fillOpacity) || 0.9 },
        { opacity: parseFloat(settings.fillOpacity / 1.20) || 0.7 }
    ]};

    console.log("chart settings", settings);

    var data = {};

    var ticks = settings.labelField ? [] : null;

    var hidden_series = $(".chart-series :checkbox:not(:checked)").map(function () {
        return $(this).val();
    });

    $.each(res['rows'], function (i, item) {
        $.each(item, function (j, cell) {
            if (isNumber(cell) && $.inArray(j, hidden_series) === -1) {
                if (settings.pie) {
                    if (!data.hasOwnProperty(j)) data[j] = 0;
                    data[j] += parseFloat(cell);
                } else {
                    if (!data.hasOwnProperty(j)) {
                        data[j] = [];
                    }

                    data[j].push([i, parseFloat(cell)]);

                    if (settings.labelField && res['rows'][i][settings.labelField] !== undefined) {
                        ticks.push([i, res['rows'][i][settings.labelField]]);
                    }
                }
            }
        });
    });

    // map data for charting
    var map_data = $.map(data, function (v, k) {
        if (settings.pie) k = k + " (" + v + ")";
        return {label: k, data: v};
    });

    console.log("map_data:", map_data);

    // show warning if there is not enough data to draw a chart
    if (map_data.length == 0) {
        $el.addAlert("Not enough data to draw a chart!");
        return;
    }

    var chart_settings = {
            series: {
                stack: settings.stack,
                lines: {
                    show: settings.line,
                    fill: settings.fill,
                    fillColor: settings.fillColor,
                    lineWidth: settings.lineWidth,
                    steps: settings.lineSteps,
                    stack: 1
                },
                points: {
                    show: settings.points,
                    lineWidth: settings.lineWidth,
                    fill: true,
                    stack: 2
                },
                bars: {
                    show: settings.bar,
                    barWidth: 0.8,
                    lineWidth: settings.lineWidth,
                    fillColor: settings.fillColor,
                    align: "center"
                },
                splines: {
                    show: settings.curvedLine,
                    fill: settings.fill,
                    fillColor: settings.fillColor,
                    tension: 0.4,
                    lineWidth: settings.lineWidth
                },
                pie: {
                    show: settings.pie,
                    innerRadius: settings.innerRadius,
                    radius: 0.8,
                    highlight: {
                        opacity: 0.05
                    }
                },
                shadowSize: 0
            },
        grid: {
            autoHighlight: true,
            hoverable: true,
            clickable: false,
            tickColor: "#e8e8e8",
            borderWidth: 1,
            borderColor: "#ccc"
            //            backgroundColor: { colors: ["#999", "#000"] }
        },
        legend: {
            noColumns: settings.legendCols,
            show: settings.legend
        },
        colors: datazen.getColors(),
        xaxis: {
            tickDecimals: 0,
            ticks: ticks,
            labelWidth: 30
        },
        yaxis: {tickDecimals: 0, autoscaleMargin: 0.1}
    };

    // plot chart
    context.plot = $.plot($el, map_data, chart_settings);

    // get y axis position
    var yaxis_left = $(".flot-y-axis>div").position().left;

    // add title
    $el.append($("<div>")
        .addClass("chart-title badge")
        .css("left", yaxis_left + 20) // relative to y axis
        .text($("#chartTitle").val()));

    var previousPoint = null;
    $el.bind("plothover", function (event, pos, item) {
        if (item) {
            if (previousPoint != item.dataIndex) {

                previousPoint = item.dataIndex;

                $(".tooltip").remove();
                var x = item.datapoint[0].toFixed(2),
                    y = item.datapoint[1].toFixed(2);

                var text = "Value: ";
                text += y;
                if (datazen.data['rows'][item.dataIndex][settings.labelField] !== undefined) {
                    text += " (" + datazen.data['rows'][item.dataIndex][settings.labelField] + ")";
                }

                showTooltip(item.pageX, item.pageY, text);
            }
        } else {
            $(".tooltip").remove();
            previousPoint = null;
        }
    });
}
datazen.exportView = function ($content) {
    var rendered_html = render('content', "#query-export", {});
    $content.html(rendered_html);
    $(".active-view .main-content").redraw();
};

datazen.executeQuery = function () {
    console.info("execute query");
    var $content = $(".active-view .query-content");
    var view = $(".active-view .content-menu-query .active").data('view');
    var viewName = view + "View";
    if (datazen.hasOwnProperty(viewName)) {
        context.activeView = view;
        datazen[viewName]($content);
    }
};

datazen.formatQuery = function () {
    var query = editor.getValue();
    $.post("/servers/format_query", {'q': query}, function (res) {
        if (res.hasOwnProperty('q') && res.q) {
            editor.setValue(res.q);
            editor.gotoLine(1);
        }
    });
};

datazen.activateLastQuery = function () {
	console.info("activeLastQuery");
    if(context.queryMode == "filter") {
//        datazen.loadFilter().setTableName(context.table);
    } else {
        $.get("/servers/%(server_id)s/last_query/%(database)s/%(table)s".format(context), function (res) {
            if (res.hasOwnProperty('query') && res.query) {
                editor.setValue(res.query);
                editor.gotoLine(1);
                // datazen.executeQuery();
            }
        }, "json");
    }
};

datazen.getTableList = function () {
    console.log("get table list");
    console.log("active table before", context.table);
    var $el = $(".active-view .tables-list");
    $el.find("li:not(.heading)").remove();
    $.get("/servers/" + context.server_id + "/tables/" + context.database, function (res) {
        $.each(res.tables, function (i, table) {
            $el.append($("<li>").append($("<a>").text(table).prop('title', table)));
        });

        if (context.initialTable) {
            $el.find("li:not(.heading)")
                .filter(function () {
                    return $(this).text() == context.initialTable;
                })
                .addClass('active');

            context.table = context.initialTable;
            context.initialTable = false;
        } else {
            context.table = $el.find("li:not(.heading):first").addClass('active').text();
        }
        datazen.showTableMeta(context.table);
        console.log("set active table", context.table);
        navigateHistory();
        datazen.activateLastQuery();
    }, "json");
};

datazen.getDBList = function () {
    console.log("get db list");
    var $el = $(".active-view .database-menu");
    $.get("/servers/%(server_id)s/databases".format(context), function (res) {
        $el.html("");
        if (res.databases) {
            $.each(res.databases, function (i, item) {
                $el.append($("<li>").append($("<a>").text(item)));
            });

            $el.parents(".db-switcher").find("a").on("click", function (e) {
                context.database = $(this).text();
//                context.table = null; // causing reset
                $(".active-view .active-db").text(context.database);
                datazen.getTableList();
                e.preventDefault();
            });

            var $ctx_el = $el.find("a").filter(function () {
                return $(this).text() == context.database;
            });

            if (context.database && $ctx_el.length > 0) {
                console.log("database in context, set it active");
                $ctx_el.click();
            } else {
                console.log("no database in context, activate first one");
                $el.find("a:first").click();
            }
        }
    });
};

datazen.createConnection = function (conn) {
    var $a = $("<a>")
        .data('server_id', conn['server_id'])
        .data('connection', conn['id'])
        .data('title', conn['server__title'])
        .text(conn['server__title'])
        .append($("<span>").addClass("glyphicon glyphicon-remove"));
    return $("<li>").append($a);
};

datazen.getConnectionList = function () {
    console.log("get connection list");
    $.get("/connections/list", function (res) {
        context.connections = res['conns'];
        var $conns = $(".tab-bar .connections");
        $conns.find("li:not(.action-tab)").remove();
        $.each(context.connections, function (i, conn) {
            $conns.find(".action-tab").before(datazen.createConnection(conn));
        });
        $conns.show();
        var $active_connection = $conns.find("a").filter(function () {
            if ($(this).data("connection") == context.connection) {
                return $(this);
            }
        });
        if (context.connection && $active_connection.length > 0) {
            console.log("found active connection:", $active_connection.data("connection"));
            $active_connection.click();
        } else {
            console.log("no active connection found, activating first connection");
            console.log("conn bug", $conns.find("a:first"));
            $conns.find("a:first").trigger("click");
        }
    }, "json");
};

datazen.loadEditor = function () {
    var $editor = $(".active-view .ace-editor");
    if ($editor.length > 0) {

        if($editor.is(".ace_editor")) {
            console.info("ace editor already active");
            return;
        } else {
            console.log("loading editor", $editor.get(0))
        }

        editor = ace.edit($editor.get(0));

        ace.config.loadModule('ace/ext/language_tools', function () {
            editor.setOptions({
                enableBasicAutocompletion: true,
                enableSnippets: true
            })
        });

        editor.setTheme("ace/theme/tomorrow");
        editor.getSession().setUseWrapMode(true);
        editor.getSession().setMode("ace/mode/sql");
        editor.setAnimatedScroll(true);

        editor.commands.addCommand({
            name: 'executeQuery',
            bindKey: {win: 'Ctrl-Enter', mac: 'Ctrl-Enter'},
            exec: datazen.executeQuery
        });

        editor.commands.addCommand({
            name: 'formatQuery',
            bindKey: {win: 'Ctrl-Shift-F', mac: 'Command-Shift-F'},
            exec: datazen.formatQuery
        });
    }
};

datazen.getQuery = function () {
    if(context.queryMode == "raw") {
        return editor.getValue();
    } else {
        return new Filter().buildQuery();
    }
}

datazen.tableMetaCache = {};
datazen.getTableMeta = function (cbk) {
    if (context.table in this.tableMetaCache) {
        console.info("fetch table meta [cache]", this.tableMetaCache[context.table])
        return cbk(this.tableMetaCache[context.table]);
    }
    var url = "/servers/%(server_id)s/meta/%(database)s/%(table)s".format(context);
    $.get(url, function (res) {
        if(res.meta) {
            console.info("fetch table meta", res.meta);
            datazen.tableMetaCache[context.table] = res.meta;
            return cbk(res.meta);
        } else {
            console.warn("empty table meta");
        }
    });
}
datazen.showTableMeta = function (table, hideOthers, cbk) {
    hideOthers = hideOthers || true;
    if (hideOthers) {
        $(".tables-list .table-meta").hide();
    }
    var $li = $(".tables-list>li:not(.heading)").filter(function () {
        return $(this).text() == table;
    });
    if($li.next().hasClass("table-meta")) {
        $li.next().toggle();
    } else {
        this.getTableMeta(function (res) {
            console.info("getTableMeta", res);
            if(res.columns) {
                var $cols = $("<ul>");
                $cols.append($("<li>").addClass("table-meta-heading").text("Columns"));
                res.columns.forEach(function (item) {
                    $cols.append($("<li>").text(item));
                });
                var $meta = $("<li>").append($cols).addClass("table-meta");
                $meta.insertAfter($li);

                if (datazen.filter) {
                    datazen.filter.setDefaultColumns(res.columns);
                    datazen.executeQuery(); // finally we can execute the query
                }

                if(cbk) {
                    cbk();
                }
            }
        });
    }
}

function Filter() {}
Filter.prototype.escape = function (str) {
    return str.replace(/[\0\x08\x09\x1a\n\r"'\\\%]/g, function (char) {
        switch (char) {
            case "\0":
                return "\\0";
            case "\x08":
                return "\\b";
            case "\x09":
                return "\\t";
            case "\x1a":
                return "\\z";
            case "\n":
                return "\\n";
            case "\r":
                return "\\r";
            case "\"":
            case "'":
            case "\\":
            case "%":
                return "\\" + char;
        }
    });
}
Filter.prototype.getWrapper = function () {
    return $(".active-view .query-mode-filter .filter");
}

Filter.prototype.getColumnsAsStr = function () {
    return this.getWrapper().find("input[name=filter-columns]").val().trim();
}
Filter.prototype.getColumns = function () {
    return this.getColumnsAsStr().split(",").map(function (el) {
        return el.trim();
    });
}

/**
 * Add condition (where)
 * @returns conditionRow
 */
Filter.prototype.addCondition = function () {
    console.log("filter -> add condition");
    var conditionRow = $(render("content", "#query-filter-condition", {}));
    conditionRow.find(".remove-condition").on("click", function () { conditionRow.remove(); });
    return this.getWrapper().append(conditionRow);
}

/**
 * Add sorting (order by)
 */
Filter.prototype.addSorting = function () {
    console.log("filter -> add sorting");
    var column = null;
    if (datazen.getColumns().length > 0) {
        column = datazen.getColumns()[0];
    }
    var sortingRow = $(render("content", "#query-filter-sorting", {column: column}));
    sortingRow.find(".remove-condition").on("click", function () { sortingRow.remove(); });
    return this.getWrapper().append(sortingRow);
}

// @todo refactor
Filter.prototype.getInputsAsDict = function ($el, prefix, keys, cbk) {
    prefix = prefix || "";
    var obj = {};
    keys.forEach(function (key) {
        obj[key] = $el.find("input[name="+prefix+key+"]").val();
        if(cbk) {
            obj[key] = cbk(obj[key]);
        }
    });
    return obj;
};
Filter.prototype.getConditions = function () {
    var that = this;
    return this.getWrapper().find(".filter-condition-row").map(function () {
        return that.getInputsAsDict($(this), "filter-condition-", ["column", "operator", "value"], that.escape);
    }).get();
}
Filter.prototype.getSorting = function () {
    var that = this;
    return this.getWrapper().find(".filter-sorting-row").map(function () {
        return that.getInputsAsDict($(this), "filter-sorting-", ["column", "order"], that.escape);
    }).get();
}
Filter.prototype.getTableName = function () {
//    return context.table;
    return this.getWrapper().find("input[name=filter-table]").val();
}

/**
 * Set value of table name
 * @param name
 */
Filter.prototype.setTableName = function (name) {
    console.log("setTableName", name);
    this.getWrapper().find("input[name=filter-table]").val(name);
//    this.setDefaultColumns();
    // @fixme bug???
}

/**
 * Build query from Filter
 * @returns {string}
 */
Filter.prototype.buildQuery = function () {
    // filter only allows "select" query
    var s = squel.select({ autoQuoteTableNames: true, autoQuoteFieldNames: true });
    // set from table
    s.from(this.getTableName());

    if(this.getColumnsAsStr() !== "*" || this.getColumnsAsStr().length === 0) {
        this.getColumns().forEach(function (col) {
            s.field(col);
        });
    }

    this.getConditions().forEach(function (item) {
        if(item.column != "") {
            var cond = "`%(column)s` %(operator)s ?".format(item);
            console.log("where cond:", cond);
            s.where(cond, item.value);
        }
    });


    this.getSorting().forEach(function (item) {
        if(item.column != "") {
            s.order(item.column, (item.order && item.order.toLowerCase() != "desc"));
        }
    });

    // set adjustable
    s.limit(20);

    // return query string
    return s.toString();
}

/**
 * Set default filter columns
 * @param cols Column array
 */
Filter.prototype.setDefaultColumns = function (cols) {
    var $wr = this.getWrapper();
    if (cols) {
        $wr.find("input[name=filter-columns]").val(cols.join(", "));
    } else {
        console.warn("no columns passed to setDefaultColumns")
    }
}

// @todo refactor and move into filter obj
datazen.loadFilter = function (cbk) {
    var view_id = "query-filter-%s".format(context.connection);
    var data = {context: context, cols: ""};
    datazen.filter = new Filter();
    openView(".active-view .query-mode-filter", view_id, "content", "#query-filter", data, function () {
        // filter controls
        $(".active-view .filter-add-condition").on("click", function () { datazen.filter.addCondition(); });
        $(".active-view .filter-add-sorting").on("click", function () { datazen.filter.addSorting(); });
        $(".active-view .execute-query").on("click", function () { datazen.filter.buildQuery(); });
        if(cbk) { cbk(); }
    });
    return datazen.filter;
};

datazen.initContentMode = function () {

    console.log("init content mode, active view:", context.activeView);

    // set active view
    $(".active-view .content-menu-query [data-view=" + context.activeView + "]").addClass('active');

    // init ace editor
    datazen.loadEditor();

    // set active db
    $(".active-view .active-db").text(context.database);

    // load DBs
    datazen.getDBList();

    // btn tabs
    $(".active-view .btn-group .btn").on("click", function () {
        $(this).siblings().removeClass('active');
        $(this).addClass('active');
    });

    // execute query event
    $(".active-view .execute-query").on("click", function (e) {
        datazen.executeQuery();
        e.preventDefault();
    });

    // content view switcher (table, viz, export)
    $(".active-view .content-menu-query .btn").on("click", function (e) {
        context.activeView = $(".content-menu-query .active").data('view');
        navigateHistory({view: $(this).data('view')});
        datazen.executeQuery();
        e.preventDefault();
    });

    // register chart update events
    $(".active-view .main-content").on("click", ".update-chart", updateChart);
    $(".active-view .main-content").on("change", ".chart-settings input, .chart-settings select", updateChart);

    // register query formatting event
    $(".active-view .format-query").on("click", datazen.formatQuery);

    // table list actions
    $(".active-view .sidebar").on("click", ".tables-list li:not(.heading) a", function (e) {
        context.table = $(this).text();
        navigateHistory();
        $(".active-view .tables-list li.active").removeClass("active");
        $(this).parent().addClass("active");
        if (datazen.filter) {
            datazen.filter.setTableName(context.table);
        }
        datazen.showTableMeta(context.table, false, datazen.activateLastQuery);
        e.preventDefault();
    });

    // query mode swither (raw, filter)
    $(".container").on("click", ".query-mode-nav a", function (e) {
        console.log("query mode switched", $(this).data("mode"));
        $(this).tab("show");
        if ($(this).data("mode") === "raw") {
            datazen.loadEditor();
        } else if ($(this).data("mode") === "filter") {
            datazen.loadFilter(function () {
                datazen.filter.setTableName(context.table);
                datazen.filter.setDefaultColumns(datazen.getColumns());
            });
        }
        context.queryMode = $(this).data("mode");
        navigateHistory();
        datazen.activateLastQuery();
        e.preventDefault();
    });

    // activate query mode
    if(context.queryMode) {
        $(".active-view .query-mode-nav a[data-mode="+context.queryMode+"]").click();
    }

//    // execute query after everything is initialized, this must be the last call
//    datazen.executeQuery();
};

datazen.initNewConnectionMode = function () {
    $(".connect-btn").on("click", function (e) {
        var $form = $(this).parents("form");
        $.post("/connections/create", $form.serialize(), function (res) {
            if (res.errors) {
                $form.showErrors(res.errors);
            } else if (res.conn) {
                var $conn = datazen.createConnection(res.conn);
                $(".connections .action-tab").before($conn);
                $conn.find("a").click();
            } else {
                alert("Unexpected error occured")
            }
        });
        e.preventDefault();
    });
};

datazen.showError = function (msg) {
    var $el = $('.alert-modal');
    $el.find(".modal-body").text(msg);
    $el.modal();
};

var navigateHistory = function (user_params) {
    var params = {
        database: context.database,
        table: context.table,
        connection: context.connection,
        view: context.activeView,
        queryMode: context.queryMode
    }
    console.log("navigate history", params, user_params);
    jQuery.extend(params, user_params);
    History.pushState(params, null, "?" + $.param(params));
};

// register handlebars helpers
Handlebars.registerHelper('cell', function (data, cols) {
    var str = '';
    $.each(cols, function (i, col) {
        var cell = Handlebars.Utils.escapeExpression(data[col]);
        str += '<td>' + cell.substring(0, 25) + '</td>';
    });

    return new Handlebars.SafeString(str);
});

Handlebars.registerHelper('trimString', function (str) {
    return str.substring(0, 150)
});

Handlebars.registerHelper('chartColors', function () {
    return context.colors;
});

// ajax csrf support
$.ajaxSetup({
    crossDomain: false,
    beforeSend: function (xhr, settings) {
        if (!csrfSafeMethod(settings.type)) {
            xhr.setRequestHeader("X-CSRFToken", getCookie('csrftoken'));
        }
    }
});

$(function () {

    var $cnt = $(".container");

    // override defaults
    context.connection = getParameterByName('connection');

    // set other context vars
    context.database = getParameterByName('database');
    context.table = getParameterByName('table');
    context.initialTable = context.table;
    context.activeView = getParameterByName('view') || "table";
    context.queryMode = getParameterByName('queryMode') || "filter"; // filter or raw

    // load alert modal in to dom
    var alertModal = render("content", "#alert-modal");
    $("body").append(alertModal);

    // connection tab click action
    $cnt.on("click", ".connections a", function (e) {

        console.log("activate connection", $(this).data('connection'));

        // save previous sate
        var prev_conn_id = $(".connections .active a").data('connection');
        if (prev_conn_id) {
            console.log("save previous context:", $(".connections .active a").data('connection'));
            var prev_key = "conn-" + prev_conn_id;
            contextStorage[prev_key] = $.extend({}, context);
        }

        // new view id
        var view_id = "conn-" + $(this).data('connection');

        // load context
        if (view_id in contextStorage) {
            console.info("load context from cache", contextStorage[view_id]);
            context = contextStorage[view_id];
        }

        // set connection and server ids
        context.connection = $(this).data('connection');
        context.server_id = $(this).data('server_id');

        // update url
        navigateHistory();

        // show tab contents
        $(this).tab("show");

        if ($(this).data('connection')) {

            // open content view for connection
            openView(".main-wrapper", view_id, "content", "#content-body", {'data': $(this).data()},
                datazen.initContentMode);

            // activate query mode (again)
            $(".active-view .query-mode-nav .active").click();

        } else if ($(this).hasClass('create-new-connection')) {

            // open new connection view
            openView(".main-wrapper", "new-conn", "connection", "#new-connection", {},
                datazen.initNewConnectionMode);

        }
        e.preventDefault();
    });

    // remove connection action
    $cnt.on("click", ".connections .glyphicon-remove", function (e) {
        e.stopImmediatePropagation();
        var id = $(this).parents('a').data('connection');
        var $ul = $(this).parents('ul');
        var isActive = $(this).parents('li').hasClass('active');
        $(this).parents('li').remove();
        if (isActive) {
            $ul.find('a:first').click();
        }
        $.post('/connections/remove', {conn: id}, function (res) {}, 'json');
        e.preventDefault();
    });

    // load connections
    datazen.getConnectionList();

    // keyboard shortcuts
    Mousetrap.bind(['ctrl+enter'], function(e) {
        datazen.executeQuery();
        return false;
    });

});
